;;;; api/probabilities.lisp
;;;;
;;;; Author: Erik Davis

(in-package  #:qvm-app)

(defgeneric perform-probabilities (simulation-method quil num-qubits &key gate-noise measurement-noise)
  (:method (simulation-method quil num-qubits &key gate-noise measurement-noise)
    (declare (ignore gate-noise measurement-noise))
    (api-method-not-implemented-error 'perform-probabilities))
  (:documentation "Executes a program and returns the resulting probability distribution over the computational basis."))

(defmethod perform-probabilities ((simulation-method (eql 'pure-state)) quil num-qubits &key gate-noise measurement-noise)
  (let* ((qvm (%execute-quil simulation-method quil num-qubits gate-noise measurement-noise))
         (amplitudes (qvm::amplitudes qvm))
         (probabilities (make-array (length amplitudes) :element-type 'qvm:flonum)))
    (map-into probabilities #'qvm::probability (qvm::amplitudes qvm))
    (values qvm
            probabilities)))

(defmethod perform-probabilities ((simulation-method (eql 'full-density-matrix)) quil num-qubits &key gate-noise measurement-noise)
  (let* ((qvm (%execute-quil simulation-method quil num-qubits gate-noise measurement-noise)))
    (values qvm
            (qvm::density-qvm-measurement-probabilities qvm))))
